# This file was auto-generated by Fern from our API Definition.

from typing import Any, Dict, List, Optional, Tuple

import pydantic


# Flattens dicts to be of the form {"key[subkey][subkey2]": value} where value is not a dict
def traverse_query_dict(
    dict_flat: Dict[str, Any], key_prefix: Optional[str] = None
) -> List[Tuple[str, Any]]:
    result = []
    for k, v in dict_flat.items():
        key = f"{key_prefix}[{k}]" if key_prefix is not None else k
        if isinstance(v, dict):
            result.extend(traverse_query_dict(v, key))
        elif isinstance(v, list):
            for arr_v in v:
                if isinstance(arr_v, dict):
                    result.extend(traverse_query_dict(arr_v, key))
                else:
                    result.append((key, arr_v))
        else:
            result.append((key, v))
    return result


def single_query_encoder(query_key: str, query_value: Any) -> List[Tuple[str, Any]]:
    if isinstance(query_value, pydantic.BaseModel) or isinstance(query_value, dict):
        if isinstance(query_value, pydantic.BaseModel):
            obj_dict = query_value.dict(by_alias=True)
        else:
            obj_dict = query_value
        return traverse_query_dict(obj_dict, query_key)
    elif isinstance(query_value, list):
        encoded_values: List[Tuple[str, Any]] = []
        for value in query_value:
            if isinstance(value, pydantic.BaseModel) or isinstance(value, dict):
                if isinstance(value, pydantic.BaseModel):
                    obj_dict = value.dict(by_alias=True)
                elif isinstance(value, dict):
                    obj_dict = value

                encoded_values.extend(single_query_encoder(query_key, obj_dict))
            else:
                encoded_values.append((query_key, value))

        return encoded_values

    return [(query_key, query_value)]


def encode_query(query: Optional[Dict[str, Any]]) -> Optional[List[Tuple[str, Any]]]:
    if query is None:
        return None

    encoded_query = []
    for k, v in query.items():
        encoded_query.extend(single_query_encoder(k, v))
    return encoded_query



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
from .generation_info import GenerationInfo
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class MessageGenerationInspection(UniversalBaseModel):
    """
    Inspection data for message generation.
    """

    generation: GenerationInfo
    messages: typing.List[typing.Optional[str]] = pydantic.Field()
    """
    The messages that were generated
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

EventSourceDto = typing.Union[
    typing.Literal[
        "customer",
        "customer_ui",
        "human_agent",
        "human_agent_on_behalf_of_ai_agent",
        "ai_agent",
        "system",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
from .payload import Payload
import pydantic
import typing
from .invoice_data import InvoiceData
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Invoice(UniversalBaseModel):
    """
    Represents the result of evaluating a single payload in an evaluation task.

    An invoice is a comprehensive record of the evaluation results for a single payload.
    """

    payload: Payload
    checksum: str = pydantic.Field()
    """
    Checksum of the invoice content
    """

    approved: bool = pydantic.Field()
    """
    Whether the evaluation task the invoice represents has been approved
    """

    data: typing.Optional[InvoiceData] = None
    error: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error message if the evaluation failed
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
from .preparation_iteration_generations import PreparationIterationGenerations
import typing
from .guideline_proposition import GuidelineProposition
import pydantic
from .tool_call import ToolCall
from .term import Term
from .context_variable_and_value import ContextVariableAndValue
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class PreparationIteration(UniversalBaseModel):
    """
    Information about a preparation iteration.
    """

    generations: PreparationIterationGenerations
    guideline_propositions: typing.List[GuidelineProposition] = pydantic.Field()
    """
    List of guideline propositions used in preparation for this iteration
    """

    tool_calls: typing.List[ToolCall] = pydantic.Field()
    """
    List of tool calls made in preparation for this iteration
    """

    terms: typing.List[Term] = pydantic.Field()
    """
    List of terms participating in the preparation for this iteration
    """

    context_variables: typing.List[ContextVariableAndValue] = pydantic.Field()
    """
    List of context variables (and their values) that participated in the preparation for this iteration
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
from .evaluation_status_dto import EvaluationStatusDto
import datetime as dt
import typing
from .invoice import Invoice
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Evaluation(UniversalBaseModel):
    """
    An evaluation task information tracking analysis of payloads.
    """

    id: str = pydantic.Field()
    """
    Unique identifier of the evaluation to retrieve
    """

    status: EvaluationStatusDto
    progress: float = pydantic.Field()
    """
    Progress of the evaluation from 0.0 to 100.0
    """

    creation_utc: dt.datetime = pydantic.Field()
    """
    UTC timestamp when the evaluation was created
    """

    error: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error message if the evaluation failed
    """

    invoices: typing.List[Invoice]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import datetime as dt
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ContextVariableValue(UniversalBaseModel):
    """
    Represents the actual stored value for a specific customer's or tag's context.

    This could be their subscription details, feature usage history,
    preferences, or any other customer or tag information that helps
    personalize the agent's responses.
    """

    id: str = pydantic.Field()
    """
    Unique identifier for the variable value
    """

    last_modified: dt.datetime = pydantic.Field()
    """
    Timestamp of the last modification
    """

    data: typing.Optional[typing.Any] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
from .tool_parameter_type_dto import ToolParameterTypeDto
import typing
import pydantic
from .tool_parameter_dto_enum_item import ToolParameterDtoEnumItem
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ToolParameter(UniversalBaseModel):
    """
    Defines a parameter that can be passed to a tool.

    Parameters can have different types with optional constraints like enums.
    Each parameter can include a description to help users understand its purpose.
    """

    type: ToolParameterTypeDto
    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Detailed description of what the parameter does and how it should be used
    """

    enum: typing.Optional[typing.List[ToolParameterDtoEnumItem]] = pydantic.Field(
        default=None
    )
    """
    List of allowed values for string or integer parameters. If provided, the parameter value must be one of these options.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class UsageInfo(UniversalBaseModel):
    """
    Token usage information.
    """

    input_tokens: int = pydantic.Field()
    """
    Amount of token received from user over the session
    """

    output_tokens: int = pydantic.Field()
    """
    Amount of token sent to user over the session
    """

    extra: typing.Optional[typing.Dict[str, typing.Optional[int]]] = pydantic.Field(
        default=None
    )
    """
    Extra data associated with the usage information
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.service import Service
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..errors.not_found_error import NotFoundError
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..errors.service_unavailable_error import ServiceUnavailableError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.tool_service_kind_dto import ToolServiceKindDto
from ..types.sdk_service_params import SdkServiceParams
from ..types.open_api_service_params import OpenApiServiceParams
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ServicesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def retrieve(
        self, name: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> Service:
        """
        Get details about a specific service including all its tools.

        The response includes:

        - Basic service information (name, kind, URL)
        - Complete list of available tools
        - Parameter definitions for each tool

        Notes:

        - Tools list may be empty if service is still initializing
        - Parameters marked as required must be provided when using a tool
        - Enum parameters restrict inputs to the listed values

        Parameters
        ----------
        name : str
            Unique identifier for the service

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Service
            Service details including all available tools

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.services.retrieve(
            name="name",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"services/{jsonable_encoder(name)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Service,
                    parse_obj_as(
                        type_=Service,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_or_update(
        self,
        name: str,
        *,
        kind: ToolServiceKindDto,
        sdk: typing.Optional[SdkServiceParams] = OMIT,
        openapi: typing.Optional[OpenApiServiceParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Service:
        """
        Creates a new service or updates an existing one.

        For SDK services:

        - Target server must implement the Parlant SDK protocol
        - Supports bidirectional communication and streaming

        For OpenAPI services:

        - Spec must be accessible and compatible with OpenAPI 3.0
        - Limited to request/response patterns

        Common requirements:

        - Service names must be unique and kebab-case
        - URLs must include http:// or https:// scheme
        - Updates cause brief service interruption while reconnecting

        Parameters
        ----------
        name : str
            Unique identifier for the service

        kind : ToolServiceKindDto

        sdk : typing.Optional[SdkServiceParams]
            SDK service configuration parameters. Required when kind is 'sdk'.

        openapi : typing.Optional[OpenApiServiceParams]
            OpenAPI service configuration parameters. Required when kind is 'openapi'.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Service
            Service successfully created or updated. The service may take a few seconds to become fully operational as it establishes connections.

        Examples
        --------
        from parlant.client import OpenApiServiceParams, ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.services.create_or_update(
            name="name",
            kind="openapi",
            openapi=OpenApiServiceParams(
                url="https://email-service.example.com/api/v1",
                source="https://email-service.example.com/api/openapi.json",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"services/{jsonable_encoder(name)}",
            method="PUT",
            json={
                "kind": kind,
                "sdk": convert_and_respect_annotation_metadata(
                    object_=sdk, annotation=SdkServiceParams, direction="write"
                ),
                "openapi": convert_and_respect_annotation_metadata(
                    object_=openapi, annotation=OpenApiServiceParams, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Service,
                    parse_obj_as(
                        type_=Service,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, name: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Removes a service integration.

        Effects:

        - Active connections are terminated immediately
        - Service tools become unavailable to agents
        - Historical data about tool usage is preserved
        - Running operations may fail

        Parameters
        ----------
        name : str
            Unique identifier for the service

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.services.delete(
            name="name",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"services/{jsonable_encoder(name)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Service]:
        """
        Returns basic info about all registered services.

        For performance reasons, tool details are omitted from the response.
        Use the retrieve endpoint to get complete information including
        tools for a specific service.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Service]
            List of all registered services. Tool lists are not
                            included for performance - use the retrieve endpoint to get tools
                            for a specific service.

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.services.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "services/",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Service],
                    parse_obj_as(
                        type_=typing.List[Service],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncServicesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def retrieve(
        self, name: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> Service:
        """
        Get details about a specific service including all its tools.

        The response includes:

        - Basic service information (name, kind, URL)
        - Complete list of available tools
        - Parameter definitions for each tool

        Notes:

        - Tools list may be empty if service is still initializing
        - Parameters marked as required must be provided when using a tool
        - Enum parameters restrict inputs to the listed values

        Parameters
        ----------
        name : str
            Unique identifier for the service

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Service
            Service details including all available tools

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.services.retrieve(
                name="name",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"services/{jsonable_encoder(name)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Service,
                    parse_obj_as(
                        type_=Service,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_or_update(
        self,
        name: str,
        *,
        kind: ToolServiceKindDto,
        sdk: typing.Optional[SdkServiceParams] = OMIT,
        openapi: typing.Optional[OpenApiServiceParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Service:
        """
        Creates a new service or updates an existing one.

        For SDK services:

        - Target server must implement the Parlant SDK protocol
        - Supports bidirectional communication and streaming

        For OpenAPI services:

        - Spec must be accessible and compatible with OpenAPI 3.0
        - Limited to request/response patterns

        Common requirements:

        - Service names must be unique and kebab-case
        - URLs must include http:// or https:// scheme
        - Updates cause brief service interruption while reconnecting

        Parameters
        ----------
        name : str
            Unique identifier for the service

        kind : ToolServiceKindDto

        sdk : typing.Optional[SdkServiceParams]
            SDK service configuration parameters. Required when kind is 'sdk'.

        openapi : typing.Optional[OpenApiServiceParams]
            OpenAPI service configuration parameters. Required when kind is 'openapi'.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Service
            Service successfully created or updated. The service may take a few seconds to become fully operational as it establishes connections.

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient, OpenApiServiceParams

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.services.create_or_update(
                name="name",
                kind="openapi",
                openapi=OpenApiServiceParams(
                    url="https://email-service.example.com/api/v1",
                    source="https://email-service.example.com/api/openapi.json",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"services/{jsonable_encoder(name)}",
            method="PUT",
            json={
                "kind": kind,
                "sdk": convert_and_respect_annotation_metadata(
                    object_=sdk, annotation=SdkServiceParams, direction="write"
                ),
                "openapi": convert_and_respect_annotation_metadata(
                    object_=openapi, annotation=OpenApiServiceParams, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Service,
                    parse_obj_as(
                        type_=Service,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, name: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Removes a service integration.

        Effects:

        - Active connections are terminated immediately
        - Service tools become unavailable to agents
        - Historical data about tool usage is preserved
        - Running operations may fail

        Parameters
        ----------
        name : str
            Unique identifier for the service

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.services.delete(
                name="name",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"services/{jsonable_encoder(name)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Service]:
        """
        Returns basic info about all registered services.

        For performance reasons, tool details are omitted from the response.
        Use the retrieve endpoint to get complete information including
        tools for a specific service.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Service]
            List of all registered services. Tool lists are not
                            included for performance - use the retrieve endpoint to get tools
                            for a specific service.

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.services.list()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "services/",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Service],
                    parse_obj_as(
                        type_=typing.List[Service],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.term import Term
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..errors.not_found_error import NotFoundError
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class GlossaryClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_terms(
        self, agent_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Term]:
        """
        Retrieves a list of all terms in the agent's glossary.

        Returns an empty list if no terms associated to the provided agent's ID.
        Terms are returned in no guaranteed order.

        Parameters
        ----------
        agent_id : str
            Unique identifier for the agent associated with the term.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Term]
            List of all terms in the agent's glossary.

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.glossary.list_terms(
            agent_id="agent_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"agents/{jsonable_encoder(agent_id)}/terms",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Term],
                    parse_obj_as(
                        type_=typing.List[Term],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_term(
        self,
        agent_id: str,
        *,
        name: str,
        description: str,
        synonyms: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Term:
        """
        Creates a new term in the agent's glossary.

        The term will be initialized with the provided name and description, and optional synonyms.
        The term will be associated with the specified agent.
        A unique identifier will be automatically generated.

        Default behaviors:

        - `synonyms` defaults to an empty list if not provided

        Parameters
        ----------
        agent_id : str
            Unique identifier for the agent associated with the term.

        name : str
            The name of the term, e.g., 'Gas' in blockchain.

        description : str
            A detailed description of the term

        synonyms : typing.Optional[typing.Sequence[str]]
            A list of synonyms for the term, including alternate contexts if applicable.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Term
            Term successfully created. Returns the complete term object including generated ID

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.glossary.create_term(
            agent_id="agent_id",
            name="Gas",
            description="A unit in Ethereum that measures the computational effort to execute transactions or smart contracts",
            synonyms=["Transaction Fee", "Blockchain Fuel"],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"agents/{jsonable_encoder(agent_id)}/terms",
            method="POST",
            json={
                "name": name,
                "description": description,
                "synonyms": synonyms,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Term,
                    parse_obj_as(
                        type_=Term,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def retrieve_term(
        self,
        agent_id: str,
        term_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Term:
        """
        Retrieves details of a specific term by ID for a given agent.

        Parameters
        ----------
        agent_id : str
            Unique identifier for the agent associated with the term.

        term_id : str
            Unique identifier for the term

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Term
            Term details successfully retrieved. Returns the complete term object

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.glossary.retrieve_term(
            agent_id="agent_id",
            term_id="term_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"agents/{jsonable_encoder(agent_id)}/terms/{jsonable_encoder(term_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Term,
                    parse_obj_as(
                        type_=Term,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_term(
        self,
        agent_id: str,
        term_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes a term from the agent.

        Deleting a non-existent term will return 404.
        No content will be returned from a successful deletion.

        Parameters
        ----------
        agent_id : str
            Unique identifier for the agent associated with the term.

        term_id : str
            Unique identifier for the term

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.glossary.delete_term(
            agent_id="agent_id",
            term_id="term_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"agents/{jsonable_encoder(agent_id)}/terms/{jsonable_encoder(term_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_term(
        self,
        agent_id: str,
        term_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        synonyms: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Term:
        """
        Parameters
        ----------
        agent_id : str
            Unique identifier for the agent associated with the term.

        term_id : str
            Unique identifier for the term

        name : typing.Optional[str]
            The name of the term, e.g., 'Gas' in blockchain.

        description : typing.Optional[str]
            A detailed description of the term

        synonyms : typing.Optional[typing.Sequence[str]]
            A list of synonyms for the term, including alternate contexts if applicable.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Term
            Term successfully updated. Returns the updated term object

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.glossary.update_term(
            agent_id="agent_id",
            term_id="term_id",
            name="Gas",
            description="A unit in Ethereum that measures the computational effort to execute transactions or smart contracts",
            synonyms=["Transaction Fee", "Blockchain Fuel"],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"agents/{jsonable_encoder(agent_id)}/terms/{jsonable_encoder(term_id)}",
            method="PATCH",
            json={
                "name": name,
                "description": description,
                "synonyms": synonyms,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Term,
                    parse_obj_as(
                        type_=Term,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncGlossaryClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_terms(
        self, agent_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Term]:
        """
        Retrieves a list of all terms in the agent's glossary.

        Returns an empty list if no terms associated to the provided agent's ID.
        Terms are returned in no guaranteed order.

        Parameters
        ----------
        agent_id : str
            Unique identifier for the agent associated with the term.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Term]
            List of all terms in the agent's glossary.

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.glossary.list_terms(
                agent_id="agent_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"agents/{jsonable_encoder(agent_id)}/terms",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Term],
                    parse_obj_as(
                        type_=typing.List[Term],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_term(
        self,
        agent_id: str,
        *,
        name: str,
        description: str,
        synonyms: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Term:
        """
        Creates a new term in the agent's glossary.

        The term will be initialized with the provided name and description, and optional synonyms.
        The term will be associated with the specified agent.
        A unique identifier will be automatically generated.

        Default behaviors:

        - `synonyms` defaults to an empty list if not provided

        Parameters
        ----------
        agent_id : str
            Unique identifier for the agent associated with the term.

        name : str
            The name of the term, e.g., 'Gas' in blockchain.

        description : str
            A detailed description of the term

        synonyms : typing.Optional[typing.Sequence[str]]
            A list of synonyms for the term, including alternate contexts if applicable.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Term
            Term successfully created. Returns the complete term object including generated ID

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.glossary.create_term(
                agent_id="agent_id",
                name="Gas",
                description="A unit in Ethereum that measures the computational effort to execute transactions or smart contracts",
                synonyms=["Transaction Fee", "Blockchain Fuel"],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"agents/{jsonable_encoder(agent_id)}/terms",
            method="POST",
            json={
                "name": name,
                "description": description,
                "synonyms": synonyms,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Term,
                    parse_obj_as(
                        type_=Term,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def retrieve_term(
        self,
        agent_id: str,
        term_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Term:
        """
        Retrieves details of a specific term by ID for a given agent.

        Parameters
        ----------
        agent_id : str
            Unique identifier for the agent associated with the term.

        term_id : str
            Unique identifier for the term

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Term
            Term details successfully retrieved. Returns the complete term object

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.glossary.retrieve_term(
                agent_id="agent_id",
                term_id="term_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"agents/{jsonable_encoder(agent_id)}/terms/{jsonable_encoder(term_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Term,
                    parse_obj_as(
                        type_=Term,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_term(
        self,
        agent_id: str,
        term_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes a term from the agent.

        Deleting a non-existent term will return 404.
        No content will be returned from a successful deletion.

        Parameters
        ----------
        agent_id : str
            Unique identifier for the agent associated with the term.

        term_id : str
            Unique identifier for the term

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.glossary.delete_term(
                agent_id="agent_id",
                term_id="term_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"agents/{jsonable_encoder(agent_id)}/terms/{jsonable_encoder(term_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_term(
        self,
        agent_id: str,
        term_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        synonyms: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Term:
        """
        Parameters
        ----------
        agent_id : str
            Unique identifier for the agent associated with the term.

        term_id : str
            Unique identifier for the term

        name : typing.Optional[str]
            The name of the term, e.g., 'Gas' in blockchain.

        description : typing.Optional[str]
            A detailed description of the term

        synonyms : typing.Optional[typing.Sequence[str]]
            A list of synonyms for the term, including alternate contexts if applicable.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Term
            Term successfully updated. Returns the updated term object

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.glossary.update_term(
                agent_id="agent_id",
                term_id="term_id",
                name="Gas",
                description="A unit in Ethereum that measures the computational effort to execute transactions or smart contracts",
                synonyms=["Transaction Fee", "Blockchain Fuel"],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"agents/{jsonable_encoder(agent_id)}/terms/{jsonable_encoder(term_id)}",
            method="PATCH",
            json={
                "name": name,
                "description": description,
                "synonyms": synonyms,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Term,
                    parse_obj_as(
                        type_=Term,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
import datetime as dt
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Agent(UniversalBaseModel):
    """
    An agent is a specialized AI personality crafted for a specific service role.

    Agents form the basic unit of conversational customization: all behavioral configurations
    are made at the agent level.

    Use this model for representing complete agent information in API responses.
    """

    id: str = pydantic.Field()
    """
    Unique identifier for the agent
    """

    name: str = pydantic.Field()
    """
    The display name of the agent, mainly for management purposes
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Detailed description of the agent's purpose and capabilities
    """

    creation_utc: dt.datetime = pydantic.Field()
    """
    UTC timestamp of when the agent was created
    """

    max_engine_iterations: int = pydantic.Field()
    """
    Maximum number of processing iterations the agent can perform per request
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
from .connection_kind_dto import ConnectionKindDto
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class GuidelineConnectionAddition(UniversalBaseModel):
    """
    Used to add connections between Guidelines.
    """

    source: str = pydantic.Field()
    """
    `id` of guideline that is source of this connection.
    """

    target: str = pydantic.Field()
    """
    `id` of guideline that is target of this connection.
    """

    kind: ConnectionKindDto

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import datetime as dt
import pydantic
import typing
from .tool_parameter import ToolParameter
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Tool(UniversalBaseModel):
    """
    Represents a single function provided by an integrated service.

    Tools are the primary way for agents to interact with external services.
    Each tool has defined parameters and can be invoked when those parameters
    are satisfied.
    """

    creation_utc: dt.datetime = pydantic.Field()
    """
    UTC timestamp when the tool was first registered with the system
    """

    name: str = pydantic.Field()
    """
    Name of the tool
    """

    description: str = pydantic.Field()
    """
    Detailed description of the tool's purpose and behavior
    """

    parameters: typing.Dict[str, ToolParameter] = pydantic.Field()
    """
    Dictionary mapping parameter names to their definitions
    """

    required: typing.List[str] = pydantic.Field()
    """
    List of parameter names that must be provided when calling the tool
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
